#include "WaitingDialogBase.h"
#include "IconHelper.h"
#include "GlobalHelper.h"
#include "KysecMessageBox.h"
#include "Common.h"

#include "stdlib.h"
#include "stdio.h"

#include <QPalette>
#include <QtWidgets/QLabel>
#include <QMovie>
#include <QtWidgets/QMessageBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QVBoxLayout>
#include <QCloseEvent>
#include <QtWidgets/QProgressBar>
#include <QPropertyAnimation>
#include <QDebug>

WaitingDialogBase::WaitingDialogBase(int operate, QWidget *parent) :
    FlatDropWidget(parent), m_operate(operate)
{
    initUI();
    this->setWindowFlags(Qt::FramelessWindowHint|Qt::Tool );

    m_result = -1;
    m_timerInterval = 0;		// 定时器超时时间，为0表示永不超时
    m_pTimer = new QTimer(this);
    connect(m_pTimer,SIGNAL(timeout()),this,SLOT(slTimerTimeout()));

    m_pProgressBar->setValue(0);
    m_pProgressBar->setTextVisible(false);

    m_pAnimation = new QPropertyAnimation(m_pProgressBar, "value", this);
    connect(m_pAnimation, SIGNAL(finished()), this, SLOT(slAnimationFinished()));
    m_pAnimation->setDuration(1500);
    m_pAnimation->setStartValue(0);
    m_pAnimation->setEndValue(100);
    m_pAnimation->stop();

    m_pProcessEventThread = new CProcessEventThread(this);
    connect(m_pProcessEventThread,SIGNAL(sigCompleted()),this,SLOT(slClose()));

    m_bForceColse = true;
    m_bIsCanForceClose = false;
    m_bIsNeedStopAnimation = false;// 是否需要停止动画
}

/*************************************************************************
 * 功能:  析构函数
 ************************************************************************/
WaitingDialogBase::~WaitingDialogBase()
{    
    if (m_pProcessEventThread->isRunning())
    {
        m_pProcessEventThread->quit();//立即停止线程
        bool isClose = m_pProcessEventThread->wait();//无限等待
        Q_UNUSED(isClose)

    }
    //注意：不要调用下面的语句，否则会引起崩溃
    delete m_pProcessEventThread;
    if (m_pTimer->isActive())
    {
        m_pTimer->stop();
    }

    m_bIsNeedStopAnimation = true;// 是否需要停止动画
    disconnect(m_pAnimation, SIGNAL(finished()), this, SLOT(slAnimationFinished()));
    while(QAbstractAnimation::Stopped != m_pAnimation->state())
    {
        m_pAnimation->stop();
    }
}

/*************************************************************************
 * 功能:  设置显示内容
 * 参数:  text    显示内容
 * 返回:  无
 ************************************************************************/
void WaitingDialogBase::setText(QString title, QString text)
{
    m_pTitleBar->setLogoLabel(title, 14, QString("#FEFEFE"));
    m_pLabel->show();
    m_pLabel->setText(text);
}

/*************************************************************************
 * 功能:  设置定时器超时时间
 * 参数:  msec，定时器超时时间，单位：毫秒
 * 返回:  无
 ************************************************************************/
void WaitingDialogBase::setTimerInterval( int msec )
{
    m_timerInterval = msec;		// 定时器超时时间，为0表示永不超时
}

/*************************************************************************
 * 功能:  设置是否允许强制关闭对话框
 * 参数:  isCanForceClose，是否允许强制关闭对话框
 * 返回:  无
 ************************************************************************/
void WaitingDialogBase::setIsCanForceClose( bool isCanForceClose )
{
    m_bIsCanForceClose = isCanForceClose;
}

/*************************************************************************
 * 功能:  开始处理事务
 * 参数:
 * 返回:  无
 ************************************************************************/
void WaitingDialogBase::start()
{
    m_pProcessEventThread->start();
    if (m_timerInterval > 0)// 定时器超时时间，为0表示永不超时
    {
        m_pTimer->start(m_timerInterval);
    }
    m_bIsNeedStopAnimation = false;// 是否需要停止动画
    m_pAnimation->start();
    QDialog::exec();
}

/*************************************************************************
 * 功能:  处理事务函数，此函数被线程调用，子类需要继承此函数
 * 参数:  无
 * 返回:  无
 ************************************************************************/
int WaitingDialogBase::processEvent()
{
    return 0;
}

/*************************************************************************
 * 功能:  关闭窗体时触发
 * 参数:  event   关闭事件对象指针
 * 返回:  无
 ************************************************************************/
void WaitingDialogBase::closeEvent(QCloseEvent *event)
{    
    int ret = 0;
    if (m_bForceColse)
    {
        //是否允许强制关闭本窗体
        if (!m_bIsCanForceClose)
        {
            KysecMessageBox* msgBox = new KysecMessageBox(this);
            if (m_operate == 0)
                msgBox->showMsg(Msg_Warning, QString("白名单文件解析添加中,禁止关闭!"));
            else if (m_operate == 1)
                msgBox->showMsg(Msg_Warning, QString("白名单文件更新中,禁止关闭!"));
            else
                msgBox->showMsg(Msg_Warning, QString("解除文件KYSEC认证保护中,禁止关闭!"));
            event->ignore();
            return ;
        }
        ret = QMessageBox::warning(this, QString("Warning"), QString("Are waiting for the complete operation, whether to continue to wait for?"), QString("Continue to wait for (&Y)"), QString("Quit(&N)"));//正在等待完成操作,是否继续等待?
        if (ret == 0)   // 继续等待
        {
            event->ignore();
            return ;
        }

        m_result = 1;
        m_pProcessEventThread->quit();//立即停止线程
        bool isClose = m_pProcessEventThread->wait();//无限等待
        Q_UNUSED(isClose);
    }

    while (m_pTimer->isActive())
    {
        m_pTimer->stop();
    }

    m_bIsNeedStopAnimation = true;// 是否需要停止动画
    disconnect(m_pAnimation, SIGNAL(finished()), this, SLOT(slAnimationFinished()));
    while(QAbstractAnimation::Stopped != m_pAnimation->state())
    {
        m_pAnimation->stop();
    }
    connect(m_pAnimation, SIGNAL(finished()), this, SLOT(slAnimationFinished()));
    QDialog::closeEvent(event);
}

/*************************************************************************
 * 功能:  槽函数,设置结束
 * 参数:  无
 * 返回:  无
 ************************************************************************/
void WaitingDialogBase::slSetResult(int res)
{
    m_result = res;
}
/*************************************************************************
 * 功能:  槽函数,调用此函数关闭窗体
 * 参数:  无
 * 返回:  无
 ************************************************************************/
void WaitingDialogBase::slClose()
{    
    m_bForceColse = false;
    this->close();
}

/*************************************************************************
 * 功能:  槽函数,动画结束后重新开始
 * 参数:  无
 * 返回:  无
 ************************************************************************/
void WaitingDialogBase::slAnimationFinished()
{
    if(!m_bIsNeedStopAnimation)// 是否需要停止动画
    {
        m_pAnimation->stop();
        m_pAnimation->start();
    }
}

/*************************************************************************
 * 功能:  槽函数,定时器时间到
 * 参数:  无
 * 返回:  无
 ************************************************************************/
void WaitingDialogBase::slTimerTimeout()
{    
    m_result = 2;
    m_bForceColse = false;
    this->close();
}

void WaitingDialogBase::slot_BtnClicked()
{
    this->close();
}

void WaitingDialogBase::initUI()
{    
    this->setFixedSize(420, 240);

    m_pTitleBar = new TitleBar(this);
    m_pTitleBar->resize(COMMON_DIALOG_WIDTH, TITLEBAR_HEIGHT);
    m_pTitleBar->setBackgroundColor("#0181EC");
    m_pTitleBar->getMinimizeBtn()->setVisible(false);
    m_pTitleBar->getMenuBtn()->setVisible(false);
    m_pTitleBar->getReturnBtn()->setVisible(false);
    m_pTitleBar->setLogoLabel(QString("KYSEC 添加白名单"), 14, QString("#FEFEFE"));
    m_pTitleBar->setLogoIcon(":/images/logo/kysec_logo_white/24.png");
    m_pTitleBar->setCloseIcon(":/images/image/close_white.png");
    m_pTitleBar->getCloseBtn()->setObjectName("WhiteCloseBtn");
    connect(m_pTitleBar->getCloseBtn(), SIGNAL(clicked(bool)), this, SLOT(close()));

    m_pLabel = new QLabel(QString("文件添加中..."));
    QPalette palette = this->palette();
    palette.setColor(QPalette::WindowText, QColor("#484B4F"));
    QFont ft;
    ft.setPointSize(18);
    m_pLabel->setPalette(palette);
    m_pLabel->setFont(ft);
    m_pLabel->setScaledContents(true);

    m_pProgressBar = new QProgressBar(this);
    m_pProgressBar->setFixedSize(360, 8);

    m_pMainLayout = new QVBoxLayout();
    m_pMainLayout->addWidget(m_pTitleBar);
    m_pMainLayout->addSpacing(54);
    m_pMainLayout->addWidget(m_pLabel, 0, Qt::AlignHCenter);
    m_pMainLayout->addSpacing(40);
    m_pMainLayout->addWidget(m_pProgressBar, 0, Qt::AlignHCenter);
    m_pMainLayout->addSpacing(86);
    m_pMainLayout->setContentsMargins(0, 0, 0, 0);

    setLayout(m_pMainLayout);
}


CProcessEventThread::CProcessEventThread(WaitingDialogBase *pWaitingDialog)
{
    m_pWaitingDialog = pWaitingDialog;
    setTerminationEnabled(true);
}

CProcessEventThread::~CProcessEventThread()
{

}

void CProcessEventThread::run()
{   
    if(NULL == m_pWaitingDialog)
    {
        return;
    }

    m_pWaitingDialog->processEvent();
    emit sigCompleted();
    return;
}

